from abc import ABC, abstractmethod


class PasswordHasher(ABC):

    @abstractmethod
    def hash_password(self, password: str) -> str:
        pass

    @abstractmethod
    def check_password(self, password: str, hashed_password: str) -> bool:
        pass
