<h1 class="text-3xl font-bold underline">Welcome to PascalZrbAppName</h1>

<ul class="list-disc">
    <li>You can find this page source code on: <code>kebab-zrb-app-name/src/frontend/src/routes/+page.svelte</code></li>
    <li>To add navigation menu, you can visit: <code>kebab-zrb-app-name/src/frontend/src/lib/config/navData.ts</code></li>
    <li>Visit <a class="underline" href="https://kit.svelte.dev/docs/project-structure">Sveltekit Documentation</a> for more information</li>
</ul>